/*
 * Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "AguiarGopinath.hh"
#include "Interpolator2D.hh"

class AguiarGopinath2EGM : public AguiarGopinath
{
private:
  double *grid_a, *min_a1, *max_a1;

  bool refine_pass;
  
  Interpolator2D VGint, BEVint, a1_dr_int;
  
  void init_BEV();
  
  double BEV(double a1, double y) const { return BEVint.get(a1, y); };
  
  int position_a1(double a1, double y, double *quad_points) const;

  const static double epsdiff = 1e-8;

  const static double min_a_update_offset = 0.01;
  
  struct BEV_deriv_params
  {
    double y;
    const AguiarGopinath2EGM *AG;
  };

  static double BEV_deriv(double a1, void *params);
  
public:
  AguiarGopinath2EGM(model_t model_arg, double gam_arg, double beta_arg,
                     double lambda_arg, double delta_arg,
                     double convergence_tol_arg, size_t ngrid_a_arg,
                     size_t ngrid_y_arg);
  AguiarGopinath2EGM(const AguiarGopinath2EGM &arg);
  virtual ~AguiarGopinath2EGM();

  virtual double a1_dr(double a, double y) const { return a1_dr_int.get(a, y); };

  virtual void compute();

  virtual double VG(double a, double y) const { return VGint.get(a, y); };
};
