/*
 * Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "AguiarGopinath.hh"
#include "Interpolator2D.hh"

class AguiarGopinathVFI : public AguiarGopinath
{
private:
  double *grid_a;

  Interpolator2D VGint, a1_dr_int;

  const double maximization_tol;
  
  struct VG_params
  {
    double a;
    double y;
    double g;
    double *quad_points;
    const AguiarGopinathVFI *AG;
  };

  static double VG_objective(double a1, void *params);

public:
  AguiarGopinathVFI(model_t model_arg, double gam_arg, double beta_arg,
                    double lambda_arg, double delta_arg,
                    double convergence_tol_arg, size_t ngrid_a_arg,
                    size_t ngrid_y_arg);
  AguiarGopinathVFI(const AguiarGopinathVFI &arg);
  virtual ~AguiarGopinathVFI();

  virtual double a1_dr(double a, double y) const { return a1_dr_int.get(a, y); };

  virtual void compute();

  virtual double VG(double a, double y) const { return VGint.get(a, y); };
};
