/*
 * Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CohenVillemot.hh"
#include "Interpolator3D.hh"

class CohenVillemot2EGM : public CohenVillemot
{
private:
  /* Should min_a also be interpreted as a debt ceiling?
     Note that it is automatically the case in VFI, so this setting is only
     relevant for 2EGM */
  const bool debt_ceiling;

  double min_a_normal, min_a_bonus_possible;

  double *grid_a_normal, *grid_a_bonus_possible,
    *grid_a1_normal, *grid_a1_bonus_possible;

  Interpolator3D VGNint, VGCint, BEV_normal_int, BEV_bonus_possible_int, a1_dr_normal_int,
    a1_dr_bonus_possible_int;

  void init_BEV();

  double BEV(double a1, double y, double z, bool bonus_possible) const;
  
  int position_a1(double a1, double y, double z, bool bonus_possible) const;

  const static double min_a_update_offset = 0.01;

  const static double init_scale_factor = 10;

  const static double dichotomy_tol = 1e-3;
  
  struct deriv_params
  {
    double y, z;
    bool bonus_possible;
    const CohenVillemot2EGM *CV;
  };

  static double BEV_deriv(double a1, void *params);

public:
  CohenVillemot2EGM(double gam_arg, double beta_arg,
                    double lambda_arg, double delta_arg,
                    double p_arg, double q_arg, double mu_z_arg,
                    double recovery_arg, double convergence_tol_arg,
                    double min_a_arg, bool debt_ceiling_arg = false);
  CohenVillemot2EGM(const CohenVillemot2EGM &arg);
  virtual ~CohenVillemot2EGM();

  virtual double VGN(double a, double y, double z) const { return VGNint.get(a, y, z); };
  virtual double VGC(double a, double y, double z) const { return VGCint.get(a, y, z); };

  virtual double a1_dr(double a, double y, double z, bool bonus_possible) const;

  virtual void compute() throw (ConvergenceError);
};
