/*
 * Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GAUSS_QUADRATURE_HH
#define GAUSS_QUADRATURE_HH

class GaussQuadrature
{
public:
  enum gauss_t { Hermite, Legendre };

  const double sigma;
  const size_t npoints;
  const gauss_t method;
  // Only relevant for Legendre method
  const double truncation_window;

  GaussQuadrature(double sigma_arg, size_t npoints_arg, gauss_t method_arg, double truncation_window_arg = 4);
  GaussQuadrature(const GaussQuadrature &arg);
  ~GaussQuadrature();

private:
  double *points, *weights;

public:
  double weight(size_t i) const { return weights[i]; };
  double point(size_t i) const { return points[i]; };

  const double *get_points() const { return points; };
  const double *get_weights() const { return weights; };
  
private:
  /*!
    Computes abscissas (hermite_points) and weights (hermite_weights) for the
    hermite quadrature (w.r. to exp(-x^2))
  
    See Abrahamowitz & Stegun:
    - p. 782, formula 22.7.13 (recurrence relationship)
    - p. 890, formula 25.4.46 (abscissas and weights)
  */
  void compute_hermite_points();

  void compute_legendre_points();

  double factorial(size_t n) const;
};

#endif
