/*
 * Copyright (C) 2012 Sébastien Villemot <sebastien@dynare.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <iostream>

#include "CohenVillemotVFI.hh"
#include "CohenVillemot2EGM.hh"

int
main(int argc, char **argv)
{
  CohenVillemot2EGM cv(/*gam=*/ 2, /*beta=*/ 0.95,
                       /*lambda=*/ 0.1, /*delta=*/ 0.02,
                       /*p=*/ 0.015, /*q=*/ 0.05, /*mu_z=*/ 0.01,
                       /*recovery=*/ -0.25*4, /*convergence_tol=*/ 1e-3,
                       /*min_a=*/ -2.5);

  cv.compute();

  gsl_rng_env_setup(); // Read environment variables

  CohenVillemot::statistics_t st = cv.model_statistics(50);

  std::cout << "sigma(y): " << st.sigma_y << std::endl;
  std::cout << "sigma(c): " << st.sigma_c << std::endl;
  std::cout << "sigma(TB/y): " << st.sigma_TB_y << std::endl;
  std::cout << "sigma(Rs): " << st.sigma_Rs << std::endl;
  std::cout << "rho(c, y): " << st.rho_c_y << std::endl;
  std::cout << "rho(TB/y, y): " << st.rho_TB_y_y << std::endl;
  std::cout << "rho(Rs, y): " << st.rho_Rs_y << std::endl;
  std::cout << "rho(Rs, TB/y): " << st.rho_Rs_TB_y << std::endl;
  std::cout << "E(Rs): " << st.E_Rs << std::endl;
  std::cout << "Defaults (per 10,000 quarters, per year): " << st.defaults_10000 << ", " << st.defaults_10000*4/100 << "%" << std::endl;
  std::cout << "Mean debt/output (quarterly, yearly) ratio: " << st.mean_debt_output << ", " << st.mean_debt_output/4*100 << "%" << std::endl;

  return 0;
}
